@@ -26,6 +26,7 @@
 
 #include <winpr/winpr.h>
 #include <winpr/crt.h>
+#include <winpr/cast.h>
 #include <winpr/assert.h>
 #include <winpr/ssl.h>
 #include <winpr/synch.h>
@@ -82,9 +83,9 @@ static void test_peer_context_free(freerdp_peer* client, rdpContext* ctx)
 		if (context->debug_channel_thread)
 		{
 			WINPR_ASSERT(context->stopEvent);
-			SetEvent(context->stopEvent);
-			WaitForSingleObject(context->debug_channel_thread, INFINITE);
-			CloseHandle(context->debug_channel_thread);
+			(void)SetEvent(context->stopEvent);
+			(void)WaitForSingleObject(context->debug_channel_thread, INFINITE);
+			(void)CloseHandle(context->debug_channel_thread);
 		}
 
 		Stream_Free(context->s, TRUE);
@@ -93,7 +94,7 @@ static void test_peer_context_free(freerdp_peer* client, rdpContext* ctx)
 		nsc_context_free(context->nsc_context);
 
 		if (context->debug_channel)
-			WTSVirtualChannelClose(context->debug_channel);
+			(void)WTSVirtualChannelClose(context->debug_channel);
 
 		sf_peer_audin_uninit(context);
 
@@ -268,8 +269,10 @@ static BOOL test_peer_draw_background(freerdp_peer* client)
 	{
 		WLog_DBG(TAG, "Using RemoteFX codec");
 		rfx_context_set_pixel_format(context->rfx_context, colorFormat);
+
+		WINPR_ASSERT(bpp <= UINT16_MAX);
 		if (!rfx_compose_message(context->rfx_context, s, &rect, 1, rgb_data, rect.width,
-		                         rect.height, rect.width * bpp))
+		                         rect.height, (UINT32)(bpp * rect.width)))
 		{
 			goto out;
 		}
@@ -284,8 +287,10 @@ static BOOL test_peer_draw_background(freerdp_peer* client)
 	{
 		WLog_DBG(TAG, "Using NSCodec");
 		nsc_context_set_parameters(context->nsc_context, NSC_COLOR_FORMAT, colorFormat);
+
+		WINPR_ASSERT(bpp <= UINT16_MAX);
 		nsc_compose_message(context->nsc_context, s, rgb_data, rect.width, rect.height,
-		                    rect.width * bpp);
+		                    (UINT32)(bpp * rect.width));
 		const UINT32 NSCodecId = freerdp_settings_get_uint32(settings, FreeRDP_NSCodecId);
 		WINPR_ASSERT(NSCodecId <= UINT16_MAX);
 		cmd.bmp.codecID = (UINT16)NSCodecId;
@@ -400,8 +405,9 @@ static void test_peer_draw_icon(freerdp_peer* client, UINT32 x, UINT32 y)
 
 	rect.x = 0;
 	rect.y = 0;
-	rect.width = context->image->width;
-	rect.height = context->image->height;
+
+	rect.width = WINPR_ASSERTING_INT_CAST(UINT16, context->image->width);
+	rect.height = WINPR_ASSERTING_INT_CAST(UINT16, context->image->height);
 
 	const UINT32 w = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
 	const UINT32 h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
@@ -553,12 +559,11 @@ static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 	rdpUpdate* update = NULL;
 	rdpPcap* pcap_rfx = NULL;
 	pcap_record record = { 0 };
-	struct server_info* info = NULL;
 
 	WINPR_ASSERT(client);
 	WINPR_ASSERT(client->context);
 
-	info = client->ContextExtra;
+	struct server_info* info = client->ContextExtra;
 	WINPR_ASSERT(info);
 
 	s = Stream_New(NULL, 512);
@@ -610,7 +615,6 @@ fail:
 static DWORD WINAPI tf_debug_channel_thread_func(LPVOID arg)
 {
 	void* fd = NULL;
-	wStream* s = NULL;
 	void* buffer = NULL;
 	DWORD BytesReturned = 0;
 	ULONG written = 0;
@@ -627,8 +631,12 @@ static DWORD WINAPI tf_debug_channel_thread_func(LPVOID arg)
 			return 0;
 	}
 
-	s = Stream_New(NULL, 4096);
-	WTSVirtualChannelWrite(context->debug_channel, (PCHAR) "test1", 5, &written);
+	wStream* s = Stream_New(NULL, 4096);
+	if (!s)
+		goto fail;
+
+	if (!WTSVirtualChannelWrite(context->debug_channel, (PCHAR) "test1", 5, &written))
+		goto fail;
 
 	while (1)
 	{
@@ -650,15 +658,16 @@ static DWORD WINAPI tf_debug_channel_thread_func(LPVOID arg)
 
 		Stream_SetPosition(s, 0);
 
-		if (WTSVirtualChannelRead(context->debug_channel, 0, (PCHAR)Stream_Buffer(s),
+		if (WTSVirtualChannelRead(context->debug_channel, 0, Stream_BufferAs(s, char),
 		                          (ULONG)Stream_Capacity(s), &BytesReturned) == FALSE)
 		{
 			if (BytesReturned == 0)
 				break;
 
-			Stream_EnsureRemainingCapacity(s, BytesReturned);
+			if (!Stream_EnsureRemainingCapacity(s, BytesReturned))
+				break;
 
-			if (WTSVirtualChannelRead(context->debug_channel, 0, (PCHAR)Stream_Buffer(s),
+			if (WTSVirtualChannelRead(context->debug_channel, 0, Stream_BufferAs(s, char),
 			                          (ULONG)Stream_Capacity(s), &BytesReturned) == FALSE)
 			{
 				/* should not happen */
@@ -757,7 +766,7 @@ static BOOL tf_peer_post_connect(freerdp_peer* client)
 			          CreateThread(NULL, 0, tf_debug_channel_thread_func, (void*)context, 0, NULL)))
 			{
 				WLog_ERR(TAG, "Failed to create debug channel thread");
-				CloseHandle(context->stopEvent);
+				(void)CloseHandle(context->stopEvent);
 				context->stopEvent = NULL;
 				return FALSE;
 			}
@@ -788,7 +797,6 @@ static BOOL tf_peer_post_connect(freerdp_peer* client)
 static BOOL tf_peer_activate(freerdp_peer* client)
 {
 	testPeerContext* context = NULL;
-	struct server_info* info = NULL;
 	rdpSettings* settings = NULL;
 
 	WINPR_ASSERT(client);
@@ -799,7 +807,7 @@ static BOOL tf_peer_activate(freerdp_peer* client)
 	settings = client->context->settings;
 	WINPR_ASSERT(settings);
 
-	info = client->ContextExtra;
+	struct server_info* info = client->ContextExtra;
 	WINPR_ASSERT(info);
 
 	context->activated = TRUE;
@@ -892,7 +900,8 @@ static BOOL tf_peer_keyboard_event(rdpInput* input, UINT16 flags, UINT8 code)
 		if (tcontext->debug_channel)
 		{
 			ULONG written = 0;
-			WTSVirtualChannelWrite(tcontext->debug_channel, (PCHAR) "test2", 5, &written);
+			if (!WTSVirtualChannelWrite(tcontext->debug_channel, (PCHAR) "test2", 5, &written))
+				return FALSE;
 		}
 	}
 	else if (((flags & KBD_FLAGS_RELEASE) == 0) && code == RDP_SCANCODE_KEY_X) /* 'x' key */
@@ -993,9 +1002,6 @@ static int hook_peer_write_pdu(rdpTransport* transport, wStream* s)
 	UINT64 ts = 0;
 	wStream* ls = NULL;
 	UINT64 last_ts = 0;
-	const struct server_info* info = NULL;
-	freerdp_peer* client = NULL;
-	testPeerContext* peerCtx = NULL;
 	size_t offset = 0;
 	UINT32 flags = 0;
 	rdpContext* context = transport_get_context(transport);
@@ -1003,16 +1009,13 @@ static int hook_peer_write_pdu(rdpTransport* transport, wStream* s)
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(s);
 
-	client = context->peer;
+	freerdp_peer* client = context->peer;
 	WINPR_ASSERT(client);
 
-	peerCtx = (testPeerContext*)client->context;
+	testPeerContext* peerCtx = (testPeerContext*)client->context;
 	WINPR_ASSERT(peerCtx);
 	WINPR_ASSERT(peerCtx->io.WritePdu);
 
-	info = client->ContextExtra;
-	WINPR_ASSERT(info);
-
 	/* Let the client authenticate.
 	 * After that is done, we stop the normal operation and send
 	 * a previously recorded session PDU by PDU to the client.
@@ -1020,7 +1023,6 @@ static int hook_peer_write_pdu(rdpTransport* transport, wStream* s)
 	 * This is fragile and the connecting client needs to use the same
 	 * configuration as the one that recorded the session!
 	 */
-	WINPR_ASSERT(info);
 	CONNECTION_STATE state = freerdp_get_state(context);
 	if (state < CONNECTION_STATE_NEGO)
 		return peerCtx->io.WritePdu(transport, s);
@@ -1038,7 +1040,12 @@ static int hook_peer_write_pdu(rdpTransport* transport, wStream* s)
 			if ((last_ts > 0) && (ts > last_ts))
 			{
 				UINT64 diff = ts - last_ts;
-				Sleep(diff);
+				while (diff > 0)
+				{
+					UINT32 d = diff > UINT32_MAX ? UINT32_MAX : (UINT32)diff;
+					diff -= d;
+					Sleep(d);
+				}
 			}
 			last_ts = ts;
 			rc = peerCtx->io.WritePdu(transport, ls);
@@ -1061,7 +1068,6 @@ static DWORD WINAPI test_peer_mainloop(LPVOID arg)
 	DWORD count = 0;
 	DWORD status = 0;
 	testPeerContext* context = NULL;
-	struct server_info* info = NULL;
 	rdpSettings* settings = NULL;
 	rdpInput* input = NULL;
 	rdpUpdate* update = NULL;
@@ -1069,7 +1075,7 @@ static DWORD WINAPI test_peer_mainloop(LPVOID arg)
 
 	WINPR_ASSERT(client);
 
-	info = client->ContextExtra;
+	struct server_info* info = client->ContextExtra;
 	WINPR_ASSERT(info);
 
 	if (!test_peer_init(client))
@@ -1203,7 +1209,7 @@ static DWORD WINAPI test_peer_mainloop(LPVOID arg)
 		if (WTSVirtualChannelManagerCheckFileDescriptor(context->vcm) != TRUE)
 			break;
 
-		/* Handle dynamic virtual channel intializations */
+		/* Handle dynamic virtual channel initializations */
 		if (WTSVirtualChannelManagerIsChannelJoined(context->vcm, DRDYNVC_SVC_CHANNEL_NAME))
 		{
 			switch (WTSVirtualChannelManagerGetDrdynvcState(context->vcm))
@@ -1257,20 +1263,19 @@ fail:
 static BOOL test_peer_accepted(freerdp_listener* instance, freerdp_peer* client)
 {
 	HANDLE hThread = NULL;
-	struct server_info* info = NULL;
 
 	WINPR_UNUSED(instance);
 
 	WINPR_ASSERT(instance);
 	WINPR_ASSERT(client);
 
-	info = instance->info;
+	struct server_info* info = instance->info;
 	client->ContextExtra = info;
 
 	if (!(hThread = CreateThread(NULL, 0, test_peer_mainloop, (void*)client, 0, NULL)))
 		return FALSE;
 
-	CloseHandle(hThread);
+	(void)CloseHandle(hThread);
 	return TRUE;
 }
 
@@ -1322,6 +1327,8 @@ static const struct
 	const char skey[6];
 } options = { "--pcap=", "--fast", "--port=", "--local-only", "--cert=", "--key=" };
 
+WINPR_PRAGMA_DIAG_PUSH
+WINPR_PRAGMA_DIAG_IGNORED_FORMAT_NONLITERAL
 WINPR_ATTR_FORMAT_ARG(2, 0)
 static void print_entry(FILE* fp, WINPR_FORMAT_ARG const char* fmt, const char* what, size_t size)
 {
@@ -1329,8 +1336,9 @@ static void print_entry(FILE* fp, WINPR_FORMAT_ARG const char* fmt, const char*
 	strncpy(buffer, what, MIN(size, sizeof(buffer) - 1));
 	(void)fprintf(fp, fmt, buffer);
 }
+WINPR_PRAGMA_DIAG_POP
 
-static WINPR_NORETURN(void usage(const char* app, const char* invalid))
+static int usage(const char* app, const char* invalid)
 {
 	FILE* fp = stdout;
 
@@ -1343,7 +1351,7 @@ static WINPR_NORETURN(void usage(const char* app, const char* invalid))
 	print_entry(fp, "\t%s\n", options.sfast, sizeof(options.sfast));
 	print_entry(fp, "\t%s<port>\n", options.sport, sizeof(options.sport));
 	print_entry(fp, "\t%s\n", options.slocal_only, sizeof(options.slocal_only));
-	exit(-1);
+	return -1;
 }
 
 int main(int argc, char* argv[])
@@ -1375,7 +1383,7 @@ int main(int argc, char* argv[])
 			port = strtol(sport, NULL, 10);
 
 			if ((port < 1) || (port > UINT16_MAX) || (errno != 0))
-				usage(app, arg);
+				return usage(app, arg);
 		}
 		else if (strncmp(arg, options.slocal_only, sizeof(options.slocal_only)) == 0)
 			localOnly = TRUE;
@@ -1383,22 +1391,22 @@ int main(int argc, char* argv[])
 		{
 			info.test_pcap_file = &arg[sizeof(options.spcap)];
 			if (!winpr_PathFileExists(info.test_pcap_file))
-				usage(app, arg);
+				return usage(app, arg);
 		}
 		else if (strncmp(arg, options.scert, sizeof(options.scert)) == 0)
 		{
 			info.cert = &arg[sizeof(options.scert)];
 			if (!winpr_PathFileExists(info.cert))
-				usage(app, arg);
+				return usage(app, arg);
 		}
 		else if (strncmp(arg, options.skey, sizeof(options.skey)) == 0)
 		{
 			info.key = &arg[sizeof(options.skey)];
 			if (!winpr_PathFileExists(info.key))
-				usage(app, arg);
+				return usage(app, arg);
 		}
 		else
-			usage(app, arg);
+			return usage(app, arg);
 	}
 
 	WTSRegisterWtsApiFunctionTable(FreeRDP_InitWtsApi());
